﻿// ---------------------------------------------------------------------------// 
// Information Section 
// ---------------------------------------------------------------------------// 
// Terminal Block Information - Programming Helper 
// Version : 2016042800 
// 
//   ****THIS PROGRAM SHOULD _NOT_ BE RAN BY A TIMER**** 
// 
// General Information (algorithm) : 
//    Enter the name of the terminal block as an argument and get information 
//    about that block including properties and actions. 
// 
//    If you want to copy the output to the clipboard enable the text 
//    panel/LCD feature and copy the public text to your clipboard by 
//    selecting all (CTRL+A) then copying to your clipboard (CTRL+C). 
// 
//    To enable text panel/LCD feature modify boolWriteToLCD variable to 
//    true or pass the argument displayon or displayoff. You will need one 
//    text panel/LCD with [TBInfo] in it's name. 
// 
// Author(s) : Phil Allen phil@hilands.com 
// Last Edited By: phil@hilands.com 
// Copyright (C) 2016  Philip Allen 
//   This program is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. 
//   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details. 
//   You should have received a copy of the GNU Lesser General Public License along with this program.  If not, see <http://www.gnu.org/licenses/>. 
//============================================================================// 
// Variables to edit 
//============================================================================// 
bool boolWriteToLCD = true; 
string strDisplayNameContains = "[TBInfo]"; 
//============================================================================// 
// Variables used by code 
//============================================================================// 
string strBlockName = ""; 
string strDebug = ""; 
IMyTerminalBlock Block = null; 
IMyTerminalBlock DisplayBlock; 
//============================================================================// 
// Main 
//============================================================================// 
void Main(string argument) 
{ 
    // check if we have an argument, if so store the argument in strBlockName. 
    // If a new argument is not provided, strBlockName will use the previous valid argument. 
    if(argument != null || argument != "") 
    { 
        if(argument.ToLower().Trim() == "displayon") 
        { 
            Echo("Enabling Display"); 
            boolWriteToLCD = true; 
            if (DisplayBlock == null) 
                GetDisplayBlock(); 
            return; 
        } 
        else if(argument.ToLower().Trim() == "displayoff") 
        { 
            Echo("Disabling Display"); 
            boolWriteToLCD = false; 
            if (DisplayBlock != null) 
            { 
                ((IMyTextPanel)DisplayBlock).WritePublicText(""); 
                ((IMyTextPanel)DisplayBlock).ShowTextureOnScreen(); 
                ((IMyTextPanel)DisplayBlock).ShowPublicTextOnScreen(); 
                DisplayBlock = null; 
            } 
            return; 
        } 
        else if(argument.ToLower().Trim() == "settings") 
        { 
            Echo("==== Current Settings ===="); 
            if(boolWriteToLCD) 
            { 
                Echo("Display: On"); 
                if (DisplayBlock == null) 
                    GetDisplayBlock(); 
            } 
            else 
                Echo("Display: Off"); 
            return; 
        } 
        strBlockName = argument; 
    } 
    //========================================================================// 
    // Check to see if the supplied argument name matches a block. 
    //========================================================================// 
    Block = GridTerminalSystem.GetBlockWithName(strBlockName); 
    if (Block == null) 
    { 
        if (DisplayBlock == null) 
            GetDisplayBlock(); 
        Echo("No Block found with the name \""+strBlockName+"\"\n"); 
        Echo("==== Optional Arguments ===="); 
        Echo("DisplayOn - enable LCD output *"); 
        Echo("DisplayOff - disables LCD output"); 
        Echo("Settings - list settings\n"); 
        Echo("* LCD must have [TBInfo] in it's name"); 
        return; 
    } 
    //========================================================================// 
    // Gather displayblock info if bool is set 
    //========================================================================// 
    if (boolWriteToLCD) 
    { 
        if (DisplayBlock == null) 
            GetDisplayBlock(); 
    } 
    //========================================================================// 
    // Reset storage variable for data output 
    //========================================================================// 
    var refStrData = new StringBuilder(); 
    //========================================================================// 
    // Get Block Name 
    //========================================================================// 
    refStrData.AppendLine("Block CustomName: "+Block.CustomName+"\n"); 
    //refStrData.AppendLine("Block DisplayNameText: "+Block.DisplayNameText+"\n"); 
    //========================================================================// 
    // Get properties of block 
    //========================================================================// 
    List<ITerminalProperty> propertyList = new List<ITerminalProperty>(); 
    Block.GetProperties(propertyList); 
    if (propertyList.Count > 0) 
    { 
        refStrData.AppendLine("Properties Found:" + (propertyList.Count)); 
        for (int i=0; i < propertyList.Count; i++) 
        { 
            // get a list of "properties" by looking up ITerminalProperty 
            refStrData.AppendLine(i+". ID: "+propertyList[i].Id); 
            refStrData.AppendLine("   TypeName: "+propertyList[i].TypeName); 
        } 
    } 
    else 
        refStrData.AppendLine("\nNo Properties Found"); 
    refStrData.AppendLine(""); 
    //========================================================================// 
    // Get actions of block 
    //========================================================================// 
    List<ITerminalAction> actionList = new List<ITerminalAction>(); 
    Block.GetActions(actionList); 
    if (actionList.Count > 0) 
    { 
        refStrData.AppendLine("Actions Found:" + actionList.Count); 
        for (int i=0; i < actionList.Count; i++) 
        { 
            refStrData.AppendLine(i+". Name: "+actionList[i].Name); 
            refStrData.AppendLine("   ID: "+actionList[i].Id); 
            //refStrData.AppendLine("Icon: "+actionList[i].Icon); 
        } 
    } 
    else 
        refStrData.AppendLine("\nNo Actions Found"); 
    refStrData.AppendLine(""); 
    //========================================================================// 
    // Get other info 
    //========================================================================// 
    refStrData.AppendLine("Block GetType:\n"+Block.GetType().ToString()+"\n"); 
    refStrData.AppendLine("Block Definition:\n"+Block.BlockDefinition+"\n"); //IMyCubeBlock 
    refStrData.AppendLine("Block DetailedInfo:\n"+Block.DetailedInfo+"\n"); //IMyTerminalBlock 
    refStrData.AppendLine("Block OwnerId: "+Block.OwnerId); 
    refStrData.AppendLine("Block CubeGrid: "+Block.CubeGrid); 
    //========================================================================// 
    // Find all Connected Grids 
    //========================================================================// 
    List<String> lGrid = new List<String>(); 
    List<IMyTerminalBlock> lBlocks = new List<IMyTerminalBlock>(); 
    GridTerminalSystem.GetBlocksOfType<IMyTerminalBlock>(lBlocks); 
    if (lBlocks.Count > 0) 
    { 
        for (int i=0; i <lBlocks.Count; i++) 
        { 
            if (!lGrid.Contains(lBlocks[i].CubeGrid.ToString())) 
                lGrid.Add(lBlocks[i].CubeGrid.ToString()); 
        } 
        if(lGrid.Count > 0) 
        { 
            refStrData.AppendLine("Connected Grids:"); 
            for (int i=0; i <lGrid.Count; i++) 
            { 
                refStrData.AppendLine(lGrid[i]); 
            } 
        } 
    } 
    //========================================================================// 
    // Take information and display it to detailed info via echo and display if toggled on 
    //========================================================================// 
    Echo(refStrData.ToString().Trim()); 
    if (boolWriteToLCD) 
    { 
        if (DisplayBlock != null) 
        { 
            ((IMyTextPanel)DisplayBlock).WritePublicText(refStrData.ToString().Trim()); 
            ((IMyTextPanel)DisplayBlock).ShowTextureOnScreen(); 
            ((IMyTextPanel)DisplayBlock).ShowPublicTextOnScreen(); 
        } 
    } 
} 
//============================================================================// 
// End Main 
//============================================================================// 
//============================================================================// 
// Get DisplayBlock 
//============================================================================// 
void GetDisplayBlock() 
{ 
    var DisplayBlocks = new List<IMyTerminalBlock>(); 
    GridTerminalSystem.SearchBlocksOfName(strDisplayNameContains, DisplayBlocks); 
    if (DisplayBlocks.Count == 1) 
    { 
        DisplayBlock = DisplayBlocks[0]; 
        Echo("Using: "+DisplayBlock.CustomName); 
    } 
    else 
        Echo("No Display Block found with \""+strDisplayNameContains+"\" in it's name or multiple matches were found."); 
} 
